@@ -1,34 +1,37 @@
 /****************************************************************************
 **
 ** Copyright (C) 2014 Klaralvdalens Datakonsult AB (KDAB).
-** Contact: http://www.qt-project.org/legal
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt3D module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -55,6 +58,9 @@
 #include <QWaitCondition>
 #include <QSurface>
 
+#if defined(QT3D_CORE_JOB_TIMING)
+#include <QElapsedTimer>
+#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -62,17 +68,18 @@ namespace Qt3DCore {
 
 QAspectManager::QAspectManager(QObject *parent)
     : QObject(parent)
-    , m_root(Q_NULLPTR)
+    , m_root(nullptr)
     , m_scheduler(new QScheduler(this))
     , m_jobManager(new QAspectJobManager(this))
     , m_changeArbiter(new QChangeArbiter(this))
     , m_serviceLocator(new QServiceLocator())
+    , m_waitForEndOfSimulationLoop(0)
     , m_waitForEndOfExecLoop(0)
     , m_waitForQuit(0)
 {
     qRegisterMetaType<QSurface *>("QSurface*");
-    m_runMainLoop.fetchAndStoreOrdered(0);
-    m_terminated.fetchAndStoreOrdered(0);
+    m_runSimulationLoop.fetchAndStoreOrdered(0);
+    m_runMainLoop.fetchAndStoreOrdered(1);
     qCDebug(Aspects) << Q_FUNC_INFO;
 }
 
@@ -83,11 +90,55 @@ QAspectManager::~QAspectManager()
     delete m_scheduler;
 }
 
-bool QAspectManager::isShuttingDown() const
+void QAspectManager::enterSimulationLoop()
 {
-    return !m_runMainLoop.load();
+    qCDebug(Aspects) << Q_FUNC_INFO;
+    m_runSimulationLoop.fetchAndStoreOrdered(1);
 }
 
+void QAspectManager::exitSimulationLoop()
+{
+    qCDebug(Aspects) << Q_FUNC_INFO;
+
+    // If this fails, simulation loop is already exited so nothing to do
+    if (!m_runSimulationLoop.testAndSetOrdered(1, 0)) {
+        qCDebug(Aspects) << "Simulation loop was not running. Nothing to do";
+        return;
+    }
+
+    QAbstractFrameAdvanceService *frameAdvanceService =
+            m_serviceLocator->service<QAbstractFrameAdvanceService>(QServiceLocator::FrameAdvanceService);
+    if (frameAdvanceService)
+        frameAdvanceService->stop();
+
+    // Give any aspects a chance to unqueue any asynchronous work they
+    // may have scheduled that would otherwise potentially deadlock or
+    // cause races. For example, the QLogicAspect queues up a vector of
+    // QNodeIds to be processed by a callback on the main thread. However,
+    // if we don't unqueue this work and release its semaphore, the logic
+    // aspect would cause a deadlock when trying to exit the inner loop.
+    // This is because we call this function from the main thread and the
+    // logic aspect is waiting for the main thread to execute the
+    // QLogicComponent::onFrameUpdate() callback.
+    for (QAbstractAspect *aspect : qAsConst(m_aspects))
+        aspect->d_func()->onEngineAboutToShutdown();
+
+    // Wait until the simulation loop is fully exited and the aspects are done
+    // processing any final changes and have had onEngineShutdown() called on them
+    m_waitForEndOfSimulationLoop.acquire(1);
+}
+
+bool QAspectManager::isShuttingDown() const
+{
+    return !m_runSimulationLoop.load();
+}
+
+/*!
+    \internal
+
+    Called by the QAspectThread's run() method immediately after the manager
+    has been created
+*/
 void QAspectManager::initialize()
 {
     qCDebug(Aspects) << Q_FUNC_INFO;
@@ -96,18 +147,23 @@ void QAspectManager::initialize()
     m_changeArbiter->initialize(m_jobManager);
 }
 
+/*!
+    \internal
+
+    Called by the QAspectThread's run() method immediately after the manager's
+    exec() function has returned.
+*/
 void QAspectManager::shutdown()
 {
     qCDebug(Aspects) << Q_FUNC_INFO;
 
-    Q_FOREACH (QAbstractAspect *aspect, m_aspects) {
-        aspect->onCleanup();
+    for (QAbstractAspect *aspect : qAsConst(m_aspects))
         m_changeArbiter->unregisterSceneObserver(aspect->d_func());
-    }
+
     // Aspects must be deleted in the Thread they were created in
 }
 
-void QAspectManager::setRootEntity(Qt3DCore::QEntity *root)
+void QAspectManager::setRootEntity(Qt3DCore::QEntity *root, const QVector<Qt3DCore::QNodeCreatedChangeBasePtr> &changes)
 {
     qCDebug(Aspects) << Q_FUNC_INFO;
 
@@ -115,30 +171,18 @@ void QAspectManager::setRootEntity(Qt3DCore::QEntity *root)
         return;
 
     if (m_root) {
-        // Allow each aspect chance to cleanup any resources from this scene
-        Q_FOREACH (QAbstractAspect *aspect, m_aspects) {
-            aspect->onCleanup();
-        }
+        // TODO: Delete all backend nodes. This is to be symmetric with how
+        // we create them below in the call to setRootAndCreateNodes
     }
 
     m_root = root;
 
     if (m_root) {
-        Q_FOREACH (QAbstractAspect *aspect, m_aspects)
-            aspect->d_func()->registerAspect(m_root);
-        m_runMainLoop.fetchAndStoreOrdered(1);
+        for (QAbstractAspect *aspect : qAsConst(m_aspects))
+            aspect->d_func()->setRootAndCreateNodes(m_root, changes);
     }
 }
 
-// Should be called after aspects are registered
-void QAspectManager::setData(const QVariantMap &data)
-{
-    qCDebug(Aspects) << Q_FUNC_INFO;
-    m_data = data;
-    Q_FOREACH (QAbstractAspect *aspect, m_aspects)
-        aspect->onInitialize(m_data);
-}
-
 /*!
  * Registers a new \a aspect.
  */
@@ -146,14 +190,16 @@ void QAspectManager::registerAspect(QAbstractAspect *aspect)
 {
     qCDebug(Aspects) << "Registering aspect";
 
-    if (aspect != Q_NULLPTR) {
+    if (aspect != nullptr) {
         m_aspects.append(aspect);
         QAbstractAspectPrivate::get(aspect)->m_aspectManager = this;
         QAbstractAspectPrivate::get(aspect)->m_jobManager = m_jobManager;
         QAbstractAspectPrivate::get(aspect)->m_arbiter = m_changeArbiter;
         // Register sceneObserver with the QChangeArbiter
         m_changeArbiter->registerSceneObserver(aspect->d_func());
-        aspect->onInitialize(m_data);
+
+        // Allow the aspect to do some work now that it is registered
+        aspect->onRegistered();
     }
     else {
         qCWarning(Aspects) << "Failed to register aspect";
@@ -161,9 +207,24 @@ void QAspectManager::registerAspect(QAbstractAspect *aspect)
     qCDebug(Aspects) << "Completed registering aspect";
 }
 
-QVariantMap QAspectManager::data() const
+/*!
+ * \internal
+ *
+ * Calls QAbstractAspect::onUnregistered(), unregisters the aspect from the
+ * change arbiter and unsets the arbiter, job manager and aspect manager.
+ * Operations are performed in the reverse order to registerAspect.
+ */
+void QAspectManager::unregisterAspect(Qt3DCore::QAbstractAspect *aspect)
 {
-    return m_data;
+    qCDebug(Aspects) << "Unregistering aspect";
+    Q_ASSERT(aspect);
+    aspect->onUnregistered();
+    m_changeArbiter->unregisterSceneObserver(aspect->d_func());
+    QAbstractAspectPrivate::get(aspect)->m_arbiter = nullptr;
+    QAbstractAspectPrivate::get(aspect)->m_jobManager = nullptr;
+    QAbstractAspectPrivate::get(aspect)->m_aspectManager = nullptr;
+    m_aspects.removeOne(aspect);
+    qCDebug(Aspects) << "Completed unregistering aspect";
 }
 
 void QAspectManager::exec()
@@ -171,9 +232,9 @@ void QAspectManager::exec()
     // Gentlemen, start your engines
     QEventLoop eventLoop;
 
-    // Enter the main loop
-    while (!m_terminated.load())
-    {
+    // Enter the engine loop
+    qCDebug(Aspects) << Q_FUNC_INFO << "***** Entering main loop *****";
+    while (m_runMainLoop.load()) {
         // Process any pending events, waiting for more to arrive if queue is empty
         eventLoop.processEvents(QEventLoop::WaitForMoreEvents, 16);
 
@@ -181,21 +242,24 @@ void QAspectManager::exec()
         QAbstractFrameAdvanceService *frameAdvanceService =
                 m_serviceLocator->service<QAbstractFrameAdvanceService>(QServiceLocator::FrameAdvanceService);
 
-        // Start the frameAdvanceService if we're about to enter the running loop
+        // Start the frameAdvanceService if we're about to enter the simulation loop
         bool needsShutdown = false;
-        if (m_runMainLoop.load()) {
+        if (m_runSimulationLoop.load()) {
             needsShutdown = true;
             frameAdvanceService->start();
 
-            // We are about to enter the main loop. Give aspects a chance to do any last
+            // We are about to enter the simulation loop. Give aspects a chance to do any last
             // pieces of initialization
-            qCDebug(Aspects) << "Calling onStartup() for each aspect";
-            Q_FOREACH (QAbstractAspect *aspect, m_aspects)
-                aspect->onStartup();
+            qCDebug(Aspects) << "Calling onEngineStartup() for each aspect";
+            for (QAbstractAspect *aspect : qAsConst(m_aspects)) {
+                qCDebug(Aspects) << "\t" << aspect->objectName();
+                aspect->onEngineStartup();
+            }
+            qCDebug(Aspects) << "Done calling onEngineStartup() for each aspect";
         }
 
-        // Only enter main render loop once the renderer and other aspects are initialized
-        while (m_runMainLoop.load()) {
+        // Only enter main simulation loop once the renderer and other aspects are initialized
+        while (m_runSimulationLoop.load()) {
             qint64 t = frameAdvanceService->waitForNextFrame();
 
             // Distribute accumulated changes. This includes changes sent from the frontend
@@ -211,21 +275,37 @@ void QAspectManager::exec()
             // For each Aspect
             // Ask them to launch set of jobs for the current frame
             // Updates matrices, bounding volumes, render bins ...
+#if defined(QT3D_CORE_JOB_TIMING)
+            QElapsedTimer timer;
+            timer.start();
+#endif
             m_scheduler->scheduleAndWaitForFrameAspectJobs(t);
+#if defined(QT3D_CORE_JOB_TIMING)
+            qDebug() << "Jobs took" << timer.nsecsElapsed() / 1.0e6;
+#endif
 
             // Process any pending events
             eventLoop.processEvents();
-        }
+        } // End of simulation loop
 
         if (needsShutdown) {
+            // Process any pending changes from the frontend before we shut the aspects down
+            m_changeArbiter->syncChanges();
+
             // Give aspects a chance to perform any shutdown actions. This may include unqueuing
             // any blocking work on the main thread that could potentially deadlock during shutdown.
-            qCDebug(Aspects) << "Calling onShutdown() for each aspect";
-            Q_FOREACH (QAbstractAspect *aspect, m_aspects)
-                aspect->onShutdown();
+            qCDebug(Aspects) << "Calling onEngineShutdown() for each aspect";
+            for (QAbstractAspect *aspect : qAsConst(m_aspects)) {
+                qCDebug(Aspects) << "\t" << aspect->objectName();
+                aspect->onEngineShutdown();
+            }
+            qCDebug(Aspects) << "Done calling onEngineShutdown() for each aspect";
+
+            // Wake up the main thread which is waiting for us inside of exitSimulationLoop()
+            m_waitForEndOfSimulationLoop.release(1);
         }
-    }
-    qCDebug(Aspects) << Q_FUNC_INFO << "Exiting event loop";
+    } // End of main loop
+    qCDebug(Aspects) << Q_FUNC_INFO << "***** Exited main loop *****";
 
     m_waitForEndOfExecLoop.release(1);
     m_waitForQuit.acquire(1);
@@ -235,23 +315,17 @@ void QAspectManager::quit()
 {
     qCDebug(Aspects) << Q_FUNC_INFO;
 
+    Q_ASSERT_X(m_runSimulationLoop.load() == 0, "QAspectManagr::quit()", "Inner loop is still running");
     m_runMainLoop.fetchAndStoreOrdered(0);
-    m_terminated.fetchAndStoreOrdered(1);
-
-    // Allow the Aspect thread to proceed in case it's locked by the
-    // FrameAdvanceService <=> it is still in the running loop
-    QAbstractFrameAdvanceService *advanceFrameService = m_serviceLocator->service<QAbstractFrameAdvanceService>(QServiceLocator::FrameAdvanceService);
-    if (advanceFrameService)
-        advanceFrameService->stop();
 
     // We need to wait for the QAspectManager exec loop to terminate
     m_waitForEndOfExecLoop.acquire(1);
     m_waitForQuit.release(1);
 
-    qCDebug(Aspects) << Q_FUNC_INFO << "Exited event loop";
+    qCDebug(Aspects) << Q_FUNC_INFO << "Exiting";
 }
 
-const QList<QAbstractAspect *> &QAspectManager::aspects() const
+const QVector<QAbstractAspect *> &QAspectManager::aspects() const
 {
     return m_aspects;
 }
