@@ -1,34 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt3D module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -39,7 +42,7 @@
 #include "qthreadpooler_p.h"
 
 #include <QMutexLocker>
-
+#include <QElapsedTimer>
 #include <QDebug>
 
 QT_BEGIN_NAMESPACE
@@ -64,8 +67,23 @@ AspectTaskRunnable::~AspectTaskRunnable()
 
 void AspectTaskRunnable::run()
 {
-    if (m_job)
+    if (m_job) {
+#ifdef QT3D_JOBS_RUN_STATS
+        QAspectJobPrivate *jobD = QAspectJobPrivate::get(m_job.data());
+        if (m_pooler) {
+            jobD->m_stats.startTime = QThreadPooler::m_jobsStatTimer.nsecsElapsed();
+            jobD->m_stats.threadId = reinterpret_cast<quint64>(QThread::currentThreadId());
+        }
+#endif
         m_job->run();
+#ifdef QT3D_JOBS_RUN_STATS
+        if (m_pooler) {
+            jobD->m_stats.endTime = QThreadPooler::m_jobsStatTimer.nsecsElapsed();
+            // Add current job's stats to log output
+            QThreadPooler::addJobLogStatsEntry(jobD->m_stats);
+        }
+#endif
+    }
 
     // We could have an append sub task or something in here
     // So that a job can post sub jobs ?
@@ -91,7 +109,7 @@ SyncTaskRunnable::SyncTaskRunnable(QAbstractAspectJobManager::JobFunction func,
     : m_func(func),
       m_arg(arg),
       m_atomicCount(atomicCount),
-      m_pooler(Q_NULLPTR),
+      m_pooler(nullptr),
       m_reserved(false)
 {
 }
@@ -123,7 +141,7 @@ void SyncTaskRunnable::setDependencyHandler(DependencyHandler *handler)
 
 DependencyHandler *SyncTaskRunnable::dependencyHandler()
 {
-    return Q_NULLPTR;
+    return nullptr;
 }
 
 } // namespace Qt3DCore {
