@@ -1,34 +1,37 @@
 /****************************************************************************
 **
 ** Copyright (C) 2014 Klaralvdalens Datakonsult AB (KDAB).
-** Contact: http://www.qt-project.org/legal
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt3D module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -45,7 +48,8 @@ namespace Qt3DRender {
 namespace Render {
 
 RenderQueue::RenderQueue()
-    : m_targetRenderViewCount(0)
+    : m_noRender(false)
+    , m_targetRenderViewCount(0)
     , m_currentRenderViewCount(0)
     , m_currentWorkQueue(1)
 {
@@ -63,7 +67,15 @@ int RenderQueue::currentRenderViewCount() const
 void RenderQueue::reset()
 {
     m_currentRenderViewCount = 0;
-    Q_ASSERT(currentRenderViewCount() == 0);
+    m_targetRenderViewCount = 0;
+    m_currentWorkQueue.clear();
+    m_noRender = false;
+}
+
+void RenderQueue::setNoRender()
+{
+    Q_ASSERT(m_targetRenderViewCount == 0);
+    m_noRender = true;
 }
 
 /*!
@@ -73,6 +85,7 @@ void RenderQueue::reset()
  */
 bool RenderQueue::queueRenderView(RenderView *renderView, uint submissionOrderIndex)
 {
+    Q_ASSERT(!m_noRender);
     m_currentWorkQueue[submissionOrderIndex] = renderView;
     ++m_currentRenderViewCount;
     return isFrameQueueComplete();
@@ -93,6 +106,7 @@ QVector<RenderView *> RenderQueue::nextFrameQueue()
  */
 void RenderQueue::setTargetRenderViewCount(int targetRenderViewCount)
 {
+    Q_ASSERT(!m_noRender);
     m_targetRenderViewCount = targetRenderViewCount;
     m_currentWorkQueue.resize(targetRenderViewCount);
 }
@@ -104,7 +118,8 @@ void RenderQueue::setTargetRenderViewCount(int targetRenderViewCount)
  */
 bool RenderQueue::isFrameQueueComplete() const
 {
-    return m_targetRenderViewCount && m_targetRenderViewCount == currentRenderViewCount();
+    return (m_noRender
+            || (m_targetRenderViewCount && m_targetRenderViewCount == currentRenderViewCount()));
 }
 
 } // namespace Render
