@@ -1,34 +1,37 @@
 /****************************************************************************
 **
 ** Copyright (C) 2014 Klaralvdalens Datakonsult AB (KDAB).
-** Contact: http://www.qt-project.org/legal
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt3D module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -46,11 +49,30 @@
 
 QT_BEGIN_NAMESPACE
 
+# ifndef QT_OPENGL_3
+#  define GL_PATCH_VERTICES 36466
+#  define GL_ACTIVE_RESOURCES 0x92F5
+#  define GL_ACTIVE_UNIFORM_BLOCKS 0x8A36
+#  define GL_BUFFER_BINDING 0x9302
+#  define GL_BUFFER_DATA_SIZE 0x9303
+#  define GL_NUM_ACTIVE_VARIABLES 0x9304
+#  define GL_SHADER_STORAGE_BLOCK 0x92E6
+#  define GL_UNIFORM 0x92E1
+#  define GL_UNIFORM_BLOCK 0x92E2
+#  define GL_UNIFORM_BLOCK_INDEX 0x8A3A
+#  define GL_UNIFORM_OFFSET 0x8A3B
+#  define GL_UNIFORM_ARRAY_STRIDE 0x8A3C
+#  define GL_UNIFORM_MATRIX_STRIDE 0x8A3D
+#  define GL_UNIFORM_BLOCK_ACTIVE_UNIFORMS 0x8A42
+#  define GL_UNIFORM_BLOCK_BINDING 0x8A3F
+#  define GL_UNIFORM_BLOCK_DATA_SIZE 0x8A40
+# endif
+
 namespace Qt3DRender {
 namespace Render {
 
 GraphicsHelperGL3::GraphicsHelperGL3()
-    : m_funcs(Q_NULLPTR)
+    : m_funcs(nullptr)
     , m_tessFuncs()
 {
 }
@@ -69,13 +91,13 @@ void GraphicsHelperGL3::initializeHelper(QOpenGLContext *context,
     }
 }
 
-void GraphicsHelperGL3::drawElementsInstanced(GLenum primitiveType,
-                                               GLsizei primitiveCount,
-                                               GLint indexType,
-                                               void *indices,
-                                               GLsizei instances,
-                                               GLint baseVertex,
-                                               GLint baseInstance)
+void GraphicsHelperGL3::drawElementsInstancedBaseVertexBaseInstance(GLenum primitiveType,
+                                                                    GLsizei primitiveCount,
+                                                                    GLint indexType,
+                                                                    void *indices,
+                                                                    GLsizei instances,
+                                                                    GLint baseVertex,
+                                                                    GLint baseInstance)
 {
     if (baseInstance != 0)
         qWarning() << "glDrawElementsInstancedBaseVertexBaseInstance is not supported with OpenGL ES 2";
@@ -101,6 +123,16 @@ void GraphicsHelperGL3::drawArraysInstanced(GLenum primitiveType,
                                    instances);
 }
 
+void GraphicsHelperGL3::drawArraysInstancedBaseInstance(GLenum primitiveType, GLint first, GLsizei count, GLsizei instances, GLsizei baseInstance)
+{
+    if (baseInstance != 0)
+        qWarning() << "glDrawArraysInstancedBaseInstance is not supported with OpenGL 3";
+    m_funcs->glDrawArraysInstanced(primitiveType,
+                                   first,
+                                   count,
+                                   instances);
+}
+
 void GraphicsHelperGL3::drawElements(GLenum primitiveType,
                                       GLsizei primitiveCount,
                                       GLint indexType,
@@ -218,6 +250,14 @@ QVector<ShaderUniformBlock> GraphicsHelperGL3::programUniformBlocks(GLuint progr
     return blocks;
 }
 
+QVector<ShaderStorageBlock> GraphicsHelperGL3::programShaderStorageBlocks(GLuint programId)
+{
+    Q_UNUSED(programId);
+    QVector<ShaderStorageBlock> blocks;
+    qWarning() << "SSBO are not supported by OpenGL 3.2 (since OpenGL 4.3)";
+    return blocks;
+}
+
 void GraphicsHelperGL3::vertexAttribDivisor(GLuint index, GLuint divisor)
 {
     Q_UNUSED(index);
@@ -239,6 +279,17 @@ void GraphicsHelperGL3::blendFunci(GLuint buf, GLenum sfactor, GLenum dfactor)
     qWarning() << "glBlendFunci() not supported by OpenGL 3.0 (since OpenGL 4.0)";
 }
 
+void GraphicsHelperGL3::blendFuncSeparatei(GLuint buf, GLenum sRGB, GLenum dRGB, GLenum sAlpha, GLenum dAlpha)
+{
+    Q_UNUSED(buf);
+    Q_UNUSED(sRGB);
+    Q_UNUSED(dRGB);
+    Q_UNUSED(sAlpha);
+    Q_UNUSED(dAlpha);
+
+    qWarning() << "glBlendFuncSeparatei() not supported by OpenGL 3.0 (since OpenGL 4.0)";
+}
+
 void GraphicsHelperGL3::alphaTest(GLenum, GLenum)
 {
     qCWarning(Render::Rendering) << "AlphaTest not available with OpenGL 3.2 core";
@@ -255,26 +306,22 @@ void GraphicsHelperGL3::depthMask(GLenum mode)
     m_funcs->glDepthMask(mode);
 }
 
-void GraphicsHelperGL3::cullFace(GLenum mode)
-{
-    m_funcs->glEnable(GL_CULL_FACE);
-    m_funcs->glCullFace(mode);
-}
-
 void GraphicsHelperGL3::frontFace(GLenum mode)
 {
     m_funcs->glFrontFace(mode);
 
 }
 
-void GraphicsHelperGL3::enableAlphaCoverage()
+void GraphicsHelperGL3::setMSAAEnabled(bool enabled)
 {
-    m_funcs->glEnable(GL_SAMPLE_ALPHA_TO_COVERAGE);
+    enabled ? m_funcs->glEnable(GL_MULTISAMPLE)
+            : m_funcs->glDisable(GL_MULTISAMPLE);
 }
 
-void GraphicsHelperGL3::disableAlphaCoverage()
+void GraphicsHelperGL3::setAlphaCoverageEnabled(bool enabled)
 {
-    m_funcs->glDisable(GL_SAMPLE_ALPHA_TO_COVERAGE);
+    enabled ? m_funcs->glEnable(GL_SAMPLE_ALPHA_TO_COVERAGE)
+            : m_funcs->glDisable(GL_SAMPLE_ALPHA_TO_COVERAGE);
 }
 
 GLuint GraphicsHelperGL3::createFrameBufferObject()
@@ -310,11 +357,11 @@ void GraphicsHelperGL3::bindFrameBufferAttachment(QOpenGLTexture *texture, const
 {
     GLenum attr = GL_DEPTH_STENCIL_ATTACHMENT;
 
-    if (attachment.m_type <= QRenderAttachment::ColorAttachment15)
-        attr = GL_COLOR_ATTACHMENT0 + attachment.m_type;
-    else if (attachment.m_type == QRenderAttachment::DepthAttachment)
+    if (attachment.m_point <= QRenderTargetOutput::Color15)
+        attr = GL_COLOR_ATTACHMENT0 + attachment.m_point;
+    else if (attachment.m_point == QRenderTargetOutput::Depth)
         attr = GL_DEPTH_ATTACHMENT;
-    else if (attachment.m_type == QRenderAttachment::StencilAttachment)
+    else if (attachment.m_point == QRenderTargetOutput::Stencil)
         attr = GL_STENCIL_ATTACHMENT;
 
     texture->bind();
@@ -359,8 +406,8 @@ void GraphicsHelperGL3::drawBuffers(GLsizei n, const int *bufs)
 
 void GraphicsHelperGL3::bindFragDataLocation(GLuint shader, const QHash<QString, int> &outputs)
 {
-    Q_FOREACH (const QString &name, outputs.keys())
-        m_funcs->glBindFragDataLocation(shader, outputs.value(name), name.toStdString().c_str());
+    for (auto it = outputs.begin(), end = outputs.end(); it != end; ++it)
+        m_funcs->glBindFragDataLocation(shader, it.value(), it.key().toStdString().c_str());
 }
 
 void GraphicsHelperGL3::bindUniform(const QVariant &v, const ShaderUniform &description)
@@ -544,6 +591,14 @@ void GraphicsHelperGL3::bindUniformBlock(GLuint programId, GLuint uniformBlockIn
     m_funcs->glUniformBlockBinding(programId, uniformBlockIndex, uniformBlockBinding);
 }
 
+void GraphicsHelperGL3::bindShaderStorageBlock(GLuint programId, GLuint shaderStorageBlockIndex, GLuint shaderStorageBlockBinding)
+{
+    Q_UNUSED(programId);
+    Q_UNUSED(shaderStorageBlockIndex);
+    Q_UNUSED(shaderStorageBlockBinding);
+    qWarning() << "SSBO are not supported by OpenGL 3.0 (since OpenGL 4.3)";
+}
+
 void GraphicsHelperGL3::bindBufferBase(GLenum target, GLuint index, GLuint buffer)
 {
     m_funcs->glBindBufferBase(target, index, buffer);
@@ -887,6 +942,14 @@ void GraphicsHelperGL3::disableClipPlane(int clipPlane)
     m_funcs->glDisable(GL_CLIP_DISTANCE0 + clipPlane);
 }
 
+void GraphicsHelperGL3::setClipPlane(int clipPlane, const QVector3D &normal, float distance)
+{
+    // deprecated
+    Q_UNUSED(clipPlane);
+    Q_UNUSED(normal);
+    Q_UNUSED(distance);
+}
+
 GLint GraphicsHelperGL3::maxClipPlaneCount()
 {
     GLint max = 0;
@@ -905,6 +968,12 @@ void GraphicsHelperGL3::disablePrimitiveRestart()
     m_funcs->glDisable(GL_PRIMITIVE_RESTART);
 }
 
+void GraphicsHelperGL3::clearBufferf(GLint drawbuffer, const QVector4D &values)
+{
+    GLfloat vec[4] = {values[0], values[1], values[2], values[3]};
+    m_funcs->glClearBufferfv(GL_COLOR, drawbuffer, vec);
+}
+
 void GraphicsHelperGL3::pointSize(bool programmable, GLfloat value)
 {
     if (programmable) {
@@ -915,6 +984,22 @@ void GraphicsHelperGL3::pointSize(bool programmable, GLfloat value)
     }
 }
 
+void GraphicsHelperGL3::enablei(GLenum cap, GLuint index)
+{
+    m_funcs->glEnablei(cap, index);
+}
+
+void GraphicsHelperGL3::disablei(GLenum cap, GLuint index)
+{
+    m_funcs->glDisablei(cap, index);
+}
+
+void GraphicsHelperGL3::setSeamlessCubemap(bool enable)
+{
+    Q_UNUSED(enable);
+    qWarning() << "GL_TEXTURE_CUBE_MAP_SEAMLESS not supported by OpenGL 3.0 (since 3.2)";
+}
+
 QSize GraphicsHelperGL3::getRenderBufferDimensions(GLuint renderBufferId)
 {
     GLint width = 0;
@@ -941,6 +1026,14 @@ QSize GraphicsHelperGL3::getTextureDimensions(GLuint textureId, GLenum target, u
     return QSize(width, height);
 }
 
+void GraphicsHelperGL3::dispatchCompute(GLuint wx, GLuint wy, GLuint wz)
+{
+    Q_UNUSED(wx);
+    Q_UNUSED(wy);
+    Q_UNUSED(wz);
+    qWarning() << "Compute Shaders are not supported by OpenGL 3.0 (since OpenGL 4.3)";
+}
+
 } // namespace Render
 } // namespace Qt3DRender
 
