@@ -1,34 +1,37 @@
 /****************************************************************************
 **
 ** Copyright (C) 2014 Klaralvdalens Datakonsult AB (KDAB).
-** Contact: http://www.qt-project.org/legal
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt3D module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -39,24 +42,95 @@
 #include "qmesh.h"
 #include "qbuffer.h"
 #include "qattribute.h"
-#include <Qt3DCore/qaxisalignedboundingbox.h>
 
-#include <Qt3DRenderer/private/renderlogging_p.h>
+#include <private/qlocale_tools_p.h>
+#include <Qt3DRender/private/qaxisalignedboundingbox_p.h>
+
+#include <Qt3DRender/private/renderlogging_p.h>
 #include <QFile>
-#include <QOpenGLBuffer>
-#include <QOpenGLShaderProgram>
 #include <QTextStream>
 #include <QVector>
+#include <QRegExp>
 
-#include <Qt3DRenderer/qgeometry.h>
-#include <Qt3DRenderer/qattribute.h>
-#include <Qt3DRenderer/qbuffer.h>
+#include <Qt3DRender/qgeometry.h>
+#include <Qt3DRender/qattribute.h>
+#include <Qt3DRender/qbuffer.h>
 
 QT_BEGIN_NAMESPACE
 
-namespace Qt3D {
+namespace Qt3DRender {
 
-inline uint qHash(const Qt3D::FaceIndices &faceIndices)
+/*
+ * A helper class to split a QByteArray and access its sections without
+ * additional memory allocations.
+ */
+class ByteArraySplitter
+{
+public:
+    explicit ByteArraySplitter(const char *begin, const char *end, char delimiter, QString::SplitBehavior splitBehavior)
+        : m_input(begin)
+    {
+        int position = 0;
+        int lastPosition = 0;
+        for (auto it = begin; it != end; ++it) {
+            if (*it == delimiter) {
+                if (position > lastPosition || splitBehavior == QString::KeepEmptyParts) { // skip multiple consecutive delimiters
+                    const Entry entry = { lastPosition, position - lastPosition };
+                    m_entries.append(entry);
+                }
+                lastPosition = position + 1;
+            }
+
+            ++position;
+        }
+
+        const Entry entry = { lastPosition, position - lastPosition };
+        m_entries.append(entry);
+    }
+
+    int size() const
+    {
+        return m_entries.size();
+    }
+
+    const char* charPtrAt(int index) const
+    {
+        return m_input + m_entries[index].start;
+    }
+
+    float floatAt(int index) const
+    {
+        return qstrntod(m_input + m_entries[index].start, m_entries[index].size, nullptr, nullptr);
+    }
+
+    int intAt(int index) const
+    {
+        return strtol(m_input + m_entries[index].start, nullptr, 10);
+    }
+
+    QString stringAt(int index) const
+    {
+        return QString::fromLatin1(m_input + m_entries[index].start, m_entries[index].size);
+    }
+
+    ByteArraySplitter splitterAt(int index, char delimiter, QString::SplitBehavior splitBehavior) const
+    {
+        return ByteArraySplitter(m_input + m_entries[index].start, m_input + m_entries[index].start + m_entries[index].size, delimiter, splitBehavior);
+    }
+
+    struct Entry
+    {
+        int start;
+        int size;
+    };
+
+private:
+    QVarLengthArray<Entry, 16> m_entries;
+    const char *m_input;
+};
+QT3D_DECLARE_TYPEINFO(Qt3DRender, ByteArraySplitter::Entry, Q_PRIMITIVE_TYPE)
+
+inline uint qHash(const FaceIndices &faceIndices)
 {
     return faceIndices.positionIndex
             + 10 * faceIndices.texCoordIndex
@@ -70,7 +144,7 @@ ObjLoader::ObjLoader()
 {
 }
 
-bool ObjLoader::load( const QString& fileName )
+bool ObjLoader::load(const QString& fileName , const QString &subMesh)
 {
     QFile file(fileName);
     if (!file.open(::QIODevice::ReadOnly|::QIODevice::Text)) {
@@ -78,7 +152,7 @@ bool ObjLoader::load( const QString& fileName )
         return false;
     }
 
-    return load(&file);
+    return load(&file, subMesh);
 }
 
 static void addFaceVertex(const FaceIndices &faceIndices,
@@ -94,7 +168,7 @@ static void addFaceVertex(const FaceIndices &faceIndices,
     }
 }
 
-bool ObjLoader::load(::QIODevice *ioDev)
+bool ObjLoader::load(::QIODevice *ioDev, const QString &subMesh)
 {
     Q_CHECK_PTR(ioDev);
     if (!ioDev->isOpen()) {
@@ -114,54 +188,102 @@ bool ObjLoader::load(::QIODevice *ioDev)
     QHash<FaceIndices, unsigned int> faceIndexMap;
     QVector<FaceIndices> faceIndexVector;
 
-    QTextStream stream(ioDev);
-    while (!stream.atEnd()) {
-        QString line = stream.readLine();
-        line = line.simplified();
+    bool skipping = false;
+    int positionsOffset = 0;
+    int normalsOffset = 0;
+    int texCoordsOffset = 0;
 
-        if (line.length() > 0 && line.at(0) != QChar::fromLatin1('#')) {
-            QTextStream lineStream(&line, QIODevice::ReadOnly);
-            QString token;
-            lineStream >> token;
+    QRegExp subMeshMatch(subMesh);
+    if (!subMeshMatch.isValid())
+        subMeshMatch.setPattern(QLatin1String("^(") + subMesh + QLatin1String(")$"));
+    Q_ASSERT(subMeshMatch.isValid());
 
-            if (token == QStringLiteral("v")) {
-                float x, y, z;
-                lineStream >> x >> y >> z;
-                positions.append(QVector3D( x, y, z ));
-            } else if (token == QStringLiteral("vt") && m_loadTextureCoords) {
-                // Process texture coordinate
-                float s,t;
-                lineStream >> s >> t;
-                texCoords.append(QVector2D(s, t));
-            } else if (token == QStringLiteral("vn")) {
-                float x, y, z;
-                lineStream >> x >> y >> z;
-                normals.append(QVector3D( x, y, z ));
-            } else if (token == QStringLiteral("f")) {
+    char lineBuffer[1024];
+    const char *line;
+    QByteArray longLine;
+    while (!ioDev->atEnd()) {
+        // try to read into lineBuffer first, if the line fits (common case) we can do this without expensive allocations
+        // if not, fall back to dynamically allocated QByteArrays
+        auto lineSize = ioDev->readLine(lineBuffer, sizeof(lineBuffer));
+        if (lineSize == sizeof(lineBuffer) - 1 && lineBuffer[lineSize - 1] != '\n') {
+            longLine = QByteArray(lineBuffer, lineSize);
+            longLine += ioDev->readLine();
+            line = longLine.constData();
+            lineSize = longLine.size();
+        } else {
+            line = lineBuffer;
+        }
+
+        if (lineSize > 0 && line[0] != '#') {
+            if (line[lineSize - 1] == '\n')
+                --lineSize; // chop newline
+
+            const ByteArraySplitter tokens(line, line + lineSize, ' ', QString::SkipEmptyParts);
+
+            if (qstrncmp(tokens.charPtrAt(0), "v ", 2) == 0) {
+                if (tokens.size() < 4) {
+                    qCWarning(Render::Io) << "Unsupported number of components in vertex";
+                } else {
+                    if (!skipping) {
+                        float x = tokens.floatAt(1);
+                        float y = tokens.floatAt(2);
+                        float z = tokens.floatAt(3);
+                        positions.append(QVector3D( x, y, z ));
+                    } else {
+                        positionsOffset++;
+                    }
+                }
+            } else if (m_loadTextureCoords && qstrncmp(tokens.charPtrAt(0), "vt ", 3) == 0) {
+                if (tokens.size() < 3) {
+                    qCWarning(Render::Io) << "Unsupported number of components in texture coordinate";
+                } else {
+                    if (!skipping) {
+                        // Process texture coordinate
+                        float s = tokens.floatAt(1);
+                        float t = tokens.floatAt(2);
+                        texCoords.append(QVector2D( s, t ));
+                    } else {
+                        texCoordsOffset++;
+                    }
+                }
+            } else if (qstrncmp(tokens.charPtrAt(0), "vn ", 3) == 0) {
+                if (tokens.size() < 4) {
+                    qCWarning(Render::Io) << "Unsupported number of components in vertex normal";
+                } else {
+                    if (!skipping) {
+                        float x = tokens.floatAt(1);
+                        float y = tokens.floatAt(2);
+                        float z = tokens.floatAt(3);
+                        normals.append(QVector3D( x, y, z ));
+                    } else {
+                        normalsOffset++;
+                    }
+                }
+            } else if (!skipping && qstrncmp(tokens.charPtrAt(0), "f ", 2) == 0) {
                 // Process face
                 ++faceCount;
-                QVector<FaceIndices> face;
-                int faceVertices = 0;
-                while (!lineStream.atEnd()) {
-                    QString faceString;
-                    lineStream >> faceString;
 
+                int faceVertices = tokens.size() - 1;
+
+                QVarLengthArray<FaceIndices, 4> face; // try to avoid allocations in the common case of triangulated data
+                face.reserve(faceVertices);
+
+                for (int i = 0; i < faceVertices; i++) {
                     FaceIndices faceIndices;
-                    QStringList indices = faceString.split(QChar::fromLatin1('/'));
+                    const ByteArraySplitter indices = tokens.splitterAt(i + 1, '/', QString::KeepEmptyParts);
                     switch (indices.size()) {
                     case 3:
-                        faceIndices.normalIndex = indices.at(2).toInt() - 1;  // fall through
+                        faceIndices.normalIndex = indices.intAt(2) - 1 - normalsOffset;  // fall through
                     case 2:
-                        faceIndices.texCoordIndex = indices.at(1).toInt() - 1; // fall through
+                        faceIndices.texCoordIndex = indices.intAt(1) - 1 - texCoordsOffset; // fall through
                     case 1:
-                        faceIndices.positionIndex = indices.at(0).toInt() - 1;
+                        faceIndices.positionIndex = indices.intAt(0) - 1 - positionsOffset;
                         break;
                     default:
                         qCWarning(Render::Io) << "Unsupported number of indices in face element";
                     }
 
                     face.append(faceIndices);
-                    ++faceVertices;
                 }
 
                 // If number of edges in face is greater than 3,
@@ -182,9 +304,20 @@ bool ObjLoader::load(::QIODevice *ioDev)
                     addFaceVertex(v1, faceIndexVector, faceIndexMap);
                     addFaceVertex(v2, faceIndexVector, faceIndexMap);
                 }
-            } // end of face
-        } // end of input line
-    } // while (!stream.atEnd())
+
+                // end of face
+            } else if (qstrncmp(tokens.charPtrAt(0), "o ", 2) == 0) {
+                if (tokens.size() < 2) {
+                    qCWarning(Render::Io) << "Missing submesh name";
+                } else {
+                    if (!subMesh.isEmpty() ) {
+                        const QString objName = tokens.stringAt(1);
+                        skipping = subMeshMatch.indexIn(objName) < 0;
+                    }
+                }
+            }
+        } // empty input line
+    } // while (!ioDev->atEnd())
 
     updateIndices(positions, normals, texCoords, faceIndexMap, faceIndexVector);
 
@@ -270,7 +403,7 @@ QGeometry *ObjLoader::geometry() const
     }
 
     QByteArray indexBytes;
-    QAttribute::DataType ty;
+    QAttribute::VertexBaseType ty;
     if (m_indices.size() < 65536) {
         // we can use USHORT
         ty = QAttribute::UnsignedShort;
@@ -315,21 +448,19 @@ void ObjLoader::updateIndices(const QVector<QVector3D> &positions,
     if (hasNormals)
         m_normals.resize(vertexCount);
 
-    foreach (const FaceIndices &faceIndices, faceIndexMap.keys()) {
-        const int i = faceIndexMap.value(faceIndices);
-
-        m_points[i] = positions[faceIndices.positionIndex];
+    for (QHash<FaceIndices, unsigned int>::const_iterator it = faceIndexMap.begin(), endIt = faceIndexMap.end(); it != endIt; ++it) {
+        m_points[it.value()] = positions[it.key().positionIndex];
         if (hasTexCoords)
-            m_texCoords[i] = std::numeric_limits<unsigned int>::max() != faceIndices.texCoordIndex ? texCoords[faceIndices.texCoordIndex] : QVector2D();
+            m_texCoords[it.value()] = std::numeric_limits<unsigned int>::max() != it.key().texCoordIndex ? texCoords[it.key().texCoordIndex] : QVector2D();
         if (hasNormals)
-            m_normals[i] = normals[faceIndices.normalIndex];
+            m_normals[it.value()] = normals[it.key().normalIndex];
     }
 
     // Now iterate over the face indices and lookup the unique vertex index
     const int indexCount = faceIndexVector.size();
     m_indices.clear();
     m_indices.reserve(indexCount);
-    foreach (const FaceIndices &faceIndices, faceIndexVector) {
+    for (const FaceIndices faceIndices : faceIndexVector) {
         const unsigned int i = faceIndexMap.value(faceIndices);
         m_indices.append(i);
     }
@@ -371,12 +502,9 @@ void ObjLoader::generateTangents( const QVector<QVector3D>& points,
     QVector<QVector3D> tan1Accum;
     QVector<QVector3D> tan2Accum;
 
-    for ( int i = 0; i < points.size(); i++ )
-    {
-        tan1Accum.append( QVector3D() );
-        tan2Accum.append( QVector3D() );
-        tangents.append( QVector4D() );
-    }
+    tan1Accum.resize(points.size());
+    tan2Accum.resize(points.size());
+    tangents.resize(points.size());
 
     // Compute the tangent vector
     for ( int i = 0; i < faces.size(); i += 3 )
@@ -411,8 +539,8 @@ void ObjLoader::generateTangents( const QVector<QVector3D>& points,
     for ( int i = 0; i < points.size(); ++i )
     {
         const QVector3D& n = normals[i];
-        QVector3D& t1 = tan1Accum[i];
-        QVector3D& t2 = tan2Accum[i];
+        const QVector3D& t1 = tan1Accum[i];
+        const QVector3D& t2 = tan2Accum[i];
 
         // Gram-Schmidt orthogonalize
         tangents[i] = QVector4D( QVector3D( t1 - QVector3D::dotProduct( n, t1 ) * n ).normalized(), 0.0f );
