@@ -1,44 +1,36 @@
 /****************************************************************************
 **
 ** Copyright (C) 2015 Klaralvdalens Datakonsult AB (KDAB).
-** Contact: http://www.qt-project.org/legal
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt3D module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include <QtTest/QtTest>
-#include <Qt3DRender/qgeometryfunctor.h>
+#include <Qt3DRender/qgeometryfactory.h>
 #include <Qt3DRender/qgeometry.h>
 
-class MeshFunctorA : public Qt3DRender::QGeometryFunctor
+class MeshFunctorA : public Qt3DRender::QGeometryFactory
 {
 public:
     MeshFunctorA()
@@ -49,10 +41,10 @@ public:
 
     Qt3DRender::QGeometry *operator ()() Q_DECL_OVERRIDE
     {
-        return Q_NULLPTR;
+        return nullptr;
     }
 
-    bool operator ==(const Qt3DRender::QGeometryFunctor &other) const Q_DECL_OVERRIDE
+    bool operator ==(const Qt3DRender::QGeometryFactory &other) const Q_DECL_OVERRIDE
     {
         return functor_cast<MeshFunctorA>(&other);
     }
@@ -60,7 +52,7 @@ public:
     QT3D_FUNCTOR(MeshFunctorA)
 };
 
-class MeshFunctorB : public Qt3DRender::QGeometryFunctor
+class MeshFunctorB : public Qt3DRender::QGeometryFactory
 {
 public:
     MeshFunctorB()
@@ -71,10 +63,10 @@ public:
 
     Qt3DRender::QGeometry *operator ()() Q_DECL_OVERRIDE
     {
-        return Q_NULLPTR;
+        return nullptr;
     }
 
-    bool operator ==(const Qt3DRender::QGeometryFunctor &other) const Q_DECL_OVERRIDE
+    bool operator ==(const Qt3DRender::QGeometryFactory &other) const Q_DECL_OVERRIDE
     {
         return functor_cast<MeshFunctorB>(&other);
     }
@@ -91,7 +83,7 @@ public:
     ~MeshFunctorASub()
     {}
 
-    bool operator ==(const Qt3DRender::QGeometryFunctor &other) const Q_DECL_OVERRIDE
+    bool operator ==(const Qt3DRender::QGeometryFactory &other) const Q_DECL_OVERRIDE
     {
         return functor_cast<MeshFunctorASub>(&other);
     }
